(*----------------------------------------------------------------------------
              CONVERTISSEUR DE NOMBRE EN CARACTERE FRANCAIS
                      http://www.developpez.com/pascal/
                        par Haypo (haypo@ifrance.com)

Ce programme convertit un nombre en caractre au format franais (de France).

------------------------------------------------------------------------------
Il supporte tous les nombres entiers relatifs et rels.

Rgle d'orthographe des nombres franais (de France) :
- le nom des dizaines ne prend de 's' que pour 'quatre vingts'.
- les nombres entre 11 et 16 (inclus) sont spciaux (de mme pour 71..76 et
  91..96).
- 'cent' ne prend de 's' qu'au pluriel s'il est  la fin.
- 'mille' ne prend jamais de 's'.

------------------------------------------------------------------------------
3 fv 2001 => Version 1.0
   - Cration du programme
   - Support des nombres entiers relatifs et rels
----------------------------------------------------------------------------*)

{$DEFINE UniteHaypo: Utilise mes units (mettez '-' devant '$DEFINE' pour}
                    { utiliser les units Turbo Pascal (Crt.tpu) }

{$IFDEF UniteHaypo}
uses Clavier, EcranTxt;
{$ELSE}
uses Crt;
{$ENDIF}

{ Nom des chifres }
function Chiffre2txt (X: Byte) : String; begin
  case X of
  0: Chiffre2txt := 'zro';
  1: Chiffre2txt := 'un';
  2: Chiffre2txt := 'deux';
  3: Chiffre2txt := 'trois';
  4: Chiffre2txt := 'quatre';
  5: Chiffre2txt := 'cinq';
  6: Chiffre2txt := 'six';
  7: Chiffre2txt := 'sept';
  8: Chiffre2txt := 'huit';
  9: Chiffre2txt := 'neuf';
  end;
end;

{ Nom des dizaines }
function Dizaine2txt (X: Byte) : String; begin
  case X of
  1: Dizaine2txt := 'dix';
  2: Dizaine2txt := 'vingt';
  3: Dizaine2txt := 'trente';
  4: Dizaine2txt := 'quarante';
  5: Dizaine2txt := 'cinquante';
  6: Dizaine2txt := 'soixante';
  7: Dizaine2txt := 'soixante-dix';
  8: Dizaine2txt := 'quatre-vingts';
  9: Dizaine2txt := 'quatre-vingts dix';
  else Dizaine2txt := '';
  end;
end;

{ Nom des nombres compris entre 10 et 20 }
function NombreDix2txt (X: Byte) : String; begin
  case X of
  0: NombreDix2txt := 'dix';
  1: NombreDix2txt := 'onze';
  2: NombreDix2txt := 'douze';
  3: NombreDix2txt := 'treize';
  4: NombreDix2txt := 'quatorze';
  5: NombreDix2txt := 'quinze';
  6: NombreDix2txt := 'seize';
  7: NombreDix2txt := 'dix-sept';
  8: NombreDix2txt := 'dix-huit';
  9: NombreDix2txt := 'dix-neuf';
  else NombreDix2txt := '';
  end;
end;

{ Nom des puissances de dix }
function NomDixPuissance (X: Byte) : String;
var TmpTxt: String[4];
begin
  case X of
  2: NomDixPuissance := 'cent';
  3: NomDixPuissance := 'mille';
  6: NomDixPuissance := 'million';
  9: NomDixPuissance := 'milliard';
  12: NomDixPuissance := 'billion';
  else begin
    Str(X,TmpTxt);
    NomDixPuissance := '10^'+TmpTxt;
  end end;
end;

{ Ajoute un espace }
procedure AjouteEspaceTxt (var txt: String); begin
  if txt<>'' then txt := txt+' ';
end;

{ Convertit en nombre entier relatif en caractre }
function Nombre2Caractere (N: LongInt) : String;
var
  txt,TmpTxt: String;

  { Puissance de dix  }
  VeutNbrMiller: boolean;     { Ecrit le nombre de milliers }
                              { (23000 -> vingt TROIS mille) }
  { Centaines }
  i: byte;
  PossibleAjouteUnS: boolean; { Est-il possible d'ajouter un S  cent ? }
  LongTxtCent: byte;          { Sauvegarde de la longueur du texte quand }
                              { 'cent' a t ajout }
begin
  { Convertit le nombre en chiffre dcimaux }
  Str (N, TmpTxt);
  PossibleAjouteUnS := true;
  VeutNbrMiller := false;
  LongTxtCent := 0;

  { Nombre ngatif ? }
  if N<0 then begin
    txt := 'moins';
    Delete (TmpTxt,1,1);
  end else txt := '';

  { Traite les chiffres suprieurs  l'unit }
  while 1<length(TmpTxt) do begin
    case (length(TmpTxt)-1) mod 3 of

    { Puissance de dix (mille, million, ...) }
    0: begin
      AjouteEspaceTxt (txt);

      { Nombre de puissance (2000 -> 2) diffrent de un ? Si oui, l'crit}
      if ((length(TmpTxt)-1=3) and ('1'<TmpTxt[1])) { Mille : que pour >1}
      or ((length(TmpTxt)-1<>3) and ('0'<TmpTxt[1])) { Autre : <> 0 }
      or (VeutNbrMiller and (TmpTxt[1]<>'0')) { ou demand }
      then begin
        txt := txt+Chiffre2txt(ord(TmpTxt[1])-Ord('0'));
        AjouteEspaceTxt (txt);
      end;

      { Ecrit le nom de la puissance }
      txt := txt+NomDixPuissance(length(TmpTxt)-1);

      { Ajoute un 's' ? }
      if (length(TmpTxt)-1<>3) and (('1'<TmpTxt[1]) or VeutNbrMiller) then txt := txt+'s';

      VeutNbrMiller := false;
    end;

    { Centaine }
    2: if TmpTxt[1]<>'0' then begin
      AjouteEspaceTxt (txt);

      { Nombre de centaine (700 -> 7) diffrent de un ? Si oui, l'crit}
      if '1'<TmpTxt[1] then begin
        txt := txt+Chiffre2txt(Ord(TmpTxt[1])-Ord('0'));
        AjouteEspaceTxt (txt);
      end;

      { Ecrit le nom de la centaine }
      txt := txt+NomDixPuissance(2);

      { Ajoute un 's' si il y a plusieurs centaines }
      { ET que cent n'est suivit d'aucun autre nombre }
      if ('1'<TmpTxt[1]) and PossibleAjouteUnS then begin
        if LongTxtCent=0 then begin
          PossibleAjouteUnS := true;
          LongTxtCent := length(txt);
        end else PossibleAjouteUnS := false;
      end;
    end;

    { Dizaine }
    1: if TmpTxt[1]<>'0' then begin
      AjouteEspaceTxt (txt);

      { Est-ce un nombre infrieur  20 ? Si oui donne ce nombre }
      { de la forme 15 -> quinze (et non 'dix cinq') }
      if TmpTxt[1]='1' then begin
        txt := txt+NombreDix2txt(ord(TmpTxt[2])-ord('0'));
        Delete (TmpTxt, 1, 1);
      end else if (TmpTxt[1]='7') or (TmpTxt[1]='9') then begin
        txt := txt+Dizaine2txt(ord(TmpTxt[1])-ord('0')-1)+' '+NombreDix2txt(ord(TmpTxt[2])-ord('0'));
        Delete (TmpTxt, 1, 1);
      end else begin
        { Sinon crit le nombre de dizaines }
        txt := txt+Dizaine2txt(ord(TmpTxt[1])-ord('0'));
        VeutNbrMiller := true;
      end;
    end end;

    { Efface le caractre trait }
    Delete (TmpTxt, 1, 1);
  end;

  { Ajoute l'unit }
  if (TmpTxt<>'') and ((N<10) or (TmpTxt[1]<>'0')) then begin
    AjouteEspaceTxt (txt);
    txt := txt+Chiffre2txt(Ord(TmpTxt[1])-Ord('0'));
  end;

  { Ajoute un 's'  cent ? }
  if LongTxtCent=length(txt) then txt := txt+'s';

  { Renvoie la chane }
  Nombre2Caractere := txt;
end;

{ Convertit en nombre rel en caractre. 'NbrChiffreVigule' est le nombre }
{ de chiffre aprs la virgule : peut tre gal  zro et doit tre infrieur }
{ ou gal  10. }
function NombreReel2Caractere (R: Real; NbrChiffreVirgule: Byte) : String;
var
  txt: String;
  DixPuissance: Real;
begin
  { Maximum : 10 chiffres aprs la virgule }
  if 10<NbrChiffreVirgule then NbrChiffreVirgule := 10;

  { Nombre ngatif ? }
  if R<0 then begin
    txt := 'moins';
    R := -R;
  end else txt := '';

  { Partie entire }
  txt := txt+Nombre2Caractere (trunc(R));

  { Partie dcimale ? }
  if (Frac(R)<>0) and (NbrChiffreVirgule<>0) then begin
    txt := txt+' virgule ';
    DixPuissance := 10;
    while 1<NbrChiffreVirgule do begin
      DixPuissance := DixPuissance*10;
      dec (NbrChiffreVirgule);
    end;
    txt := txt+Nombre2Caractere (trunc(round(Frac(R)*DixPuissance)));
  end;

  NombreReel2Caractere := txt;
end;

procedure AfficheExempleEntier (x: LongInt); begin
{$IFDEF UniteHaypo}
  EcriTnc (x,15);
  Ecrit (' >');
  EcriTc (Nombre2Caractere (x),10);
  EcriTr ('<');
{$ELSE}
  TextColor (15); Write (x); TextColor (7);
  Write (' >');
  TextColor (10); Write (Nombre2Caractere (x)); TextColor (7);
  Writeln ('<');
{$ENDIF}
end;

procedure AfficheExempleReel (x: Real; p: byte);
{$IFDEF UniteHaypo}
  var txt: String;
begin
  if 10<p then p := 10;
  Str (x:0:p,txt);
  EcriTc (txt,15);
  Ecrit (' >');
  EcriTc (NombreReel2Caractere (x,p),10);
  EcriTr ('<');
  SauteLigne;
{$ELSE}
begin
  TextColor (15); Write (x:0:p); TextColor (7);
  Write (' >');
  TextColor (10); Write (NombreReel2Caractere (x,p)); TextColor (7);
  Writeln ('<');
  Writeln;
{$ENDIF}
end;

var
  x: LongInt;
  xR: real;
  p: byte;
begin
  { Affiche quelques exemples de nombres entiers relatifs }
{$IFDEF UniteHaypo}
  EffaceEcranTxt;
  EcriTcr ('[ NOMBRES ENTIERS ]',15+3*16);
  SauteLigne;
{$ELSE}
  ClrScr;
  TextColor (15); TextBackGround (3);
  Writeln ('[ NOMBRES ENTIERS ]');
  TextColor (7); TextBackGround (0);
  Writeln;
{$ENDIF}
  AfficheExempleEntier (-1);
  AfficheExempleEntier (-1);
  AfficheExempleEntier (0);
  AfficheExempleEntier (1);
  AfficheExempleEntier (7);
  AfficheExempleEntier (10);
  AfficheExempleEntier (13);
  AfficheExempleEntier (17);
  AfficheExempleEntier (20);
  AfficheExempleEntier (26);
  AfficheExempleEntier (30);
  AfficheExempleEntier (40);
  AfficheExempleEntier (73);
  AfficheExempleEntier (100);
  AfficheExempleEntier (200);
  AfficheExempleEntier (201);
  AfficheExempleEntier (1000);
  AfficheExempleEntier (1234);
  AfficheExempleEntier (100204);
  {$IFDEF UniteHaypo} LitTouche; {$ELSE} ReadKey; {$ENDIF}

  { Affiche quelques exemples de nombres rels }

{$IFDEF UniteHaypo}
  EffaceEcranTxt;
  EcriTcr ('[ NOMBRES REELS (ici le nombre PI) ]',15+3*16);
  SauteLigne;
{$ELSE}
  ClrScr;
  TextColor (15); TextBackGround (3);
  Writeln ('[ NOMBRES REELS (ici le nombre PI) ]');
  TextColor (7); TextBackGround (0);
  Writeln;
{$ENDIF}
  xR := pi;

  p := 0; Writeln ('*** Prcision = ',p,' chiffre aprs la virgule ***');
  AfficheExempleReel (xR, p);

  p := 3; Writeln ('*** Prcision = ',p,' chiffres aprs la virgule ***');
  AfficheExempleReel (xR, p);

  p := 6; Writeln ('*** Prcision = ',p,' chiffres aprs la virgule ***');
  AfficheExempleReel (xR, p);

  p := 8; Writeln ('*** Prcision = ',p,' chiffres aprs la virgule ***');
  AfficheExempleReel (xR, p);

  p := 10; Writeln ('*** Prcision = ',p,' chiffres (MAXIMUM) aprs la virgule ***');
  AfficheExempleReel (xR, p);

  {$IFDEF UniteHaypo} LitTouche; {$ELSE} ReadKey; {$ENDIF}
end.